// STC15 Hello World
// Modified, with much respect, from work by Rick100:
// https://github.com/Rick-100/STC15F104-Hello-World
// Toggles P3.2 on and off at 1 Hz (connect an LED to see)
// Transmits "Hack the Planet" over UART at 2400 baud

#include "my_stc_.h"

/* 
put the following three lines in a file named build.bat:

sdcc -mmcs51  --iram-size 128 --xram-size 0 --code-size 4088 --verbose hello.c -o hello_bin.ihx
packihx hello_bin.ihx > hello.hex
pause

Place build.bat, this file (hello.c), and my_stc_.h in the same directory
*/
 
#define led     P3_2
#define txLine  P3_1

#define TX_HIGH txLine = 1
#define TX_LOW  txLine = 0

// sets mode of an I/O pin
// only coded to work on ports 1 and 3 so far
void setPortMode(unsigned char port,unsigned char bitNum, unsigned char mode)
{
  unsigned char tmp1 = 0;
  unsigned char tmp2 = 0;
  unsigned char tmp3 = 0;
  unsigned char tmp4 = 0;
  if(mode & 0x01){
    tmp1 = 1 << bitNum;
  }
  if(mode & 0x02){
    tmp2 = 1 << bitNum;
  }
  
  switch(port){
    case 1 :
      tmp3 = (P1M0 & ~tmp1) | tmp1;
      tmp4 = (P1M1 & ~tmp1) | tmp2;
      P1M0 = tmp3;
      P1M1 = tmp4;
      break;
    case 3 :
      tmp3 = (P3M0 & ~tmp1) | tmp1;
      tmp4 = (P3M1 & ~tmp1) | tmp2;
      P3M0 = tmp3;
      P3M1 = tmp4;
      break;
  }	
}

// 1 bit time for 2400 baud at 12 MHz
void Delay2400()
{
    __asm
        push 0x30
        push 0x31
        mov 0x30,#4
        mov 0x31,#220
NEXT:
        djnz 0x31,NEXT
        djnz 0x30,NEXT
        pop 0x31
        pop 0x30
    __endasm;
}


void sendChar(unsigned char c)	//send an ASCII character
{
  unsigned char mask = 1;  //bit mask
  unsigned char i;

  // insert two STOP BITS bits before transmitting
  Delay2400();
  Delay2400();
  TX_LOW;       // set output LOW for START BIT
  Delay2400();  // wait 1 START BIT
  
  for (i=0; i<8 ;i++){
    if (c & mask) {
      TX_HIGH;
    } else {
      TX_LOW;
    }
    mask <<= 1;
    Delay2400();
  }
  TX_HIGH;  //Restore TXDATA pin to HIGH
}

void sendCRLF(void)
{
  sendChar(13);
  sendChar(10);
}

void main()
{
  unsigned int delayCtr;
  char msg[] = "Hack the Planet";
  int c;  

  setPortMode(3,3,1); //make bit 3 on P3 a push pull output
  setPortMode(3,2,1); //make bit 2 on P3 a push pull output
	
  led = 0;

  while(1)
  {
    for(delayCtr = 2400;delayCtr > 0;delayCtr--)  //wait 1 second
    {
        Delay2400();
    }
    led = !led;  //toggle led
    c = 0;
    while (msg[c] != 0)
    {
       sendChar(msg[c]);
       c++;
    }
    sendCRLF();
  }		
}	  
